"""Holds basic metadata on the fore optics of Lux"""

# Standard library
from dataclasses import dataclass

# Third-party
import astropy.units as u
import astropy.io.fits as fits
import numpy as np

# Local
from . import config

@dataclass
class Hardware:
    """Holds basic metadata on the optics of Lux

    Args:
        mirror_diameter (float): Diameter of the Lux primary mirror
    """

    def __repr__(self):
        return "Lux Fore Optics"

    @property
    def primary_mirror_physical_diameter(self):
        """Diameter of Lux's primary mirror that will reflect light."""
        return 44.25 * u.cm

    @property
    def secondary_mirror_physical_diameter(self):
        """Diameter of Pandora's secondary mirror that will block light."""
        return 10.6 * u.cm
        
    @property
    def primary_mirror_spider_area(self):
    	"""Area primary mirror spiders that will block light."""
    	return ((18.0 * u.cm) * (0.965 * u.cm))
    	
    @property
    def number_primary_mirror_spiders(self):
    	"""Number of primary mirror spiders that will block light."""
    	return 4
    	
    @property
    def primary_mirror_effective_diameter(self):
        """Diameter of Lux's mirror. This should be the effective diameter of the primary, removing the secondary diameter and spiders."""
        A_primary = np.pi * (self.primary_mirror_physical_diameter / 2) ** 2
        A_secondary = (
            np.pi * (self.secondary_mirror_physical_diameter / 2) ** 2
        )
        A_spiders = self.number_primary_mirror_spiders * self.primary_mirror_spider_area
        r_primary = ((A_primary - A_secondary - A_spiders) / np.pi) ** 0.5
        return r_primary * 2
        
    @property
    def fore_throughput(self):
    	"""Throughput of primary and secondary (fore) optics"""
    	f = fits.open(config.DATADIR + "Fore_Optics_Throughput.fits")
    	return f[1].data["Throughput"]


# Standard library
import configparser  # noqa: E402
import logging  # noqa: E402
import os  # noqa
from glob import glob
from importlib.metadata import PackageNotFoundError, version  # noqa

# Third-party
import numpy as np  # noqa
import pandas as pd  # noqa
import astropy.io.fits as fits
import astropy.units as u
import astropy.constants as const
from astropy.modeling.models import Gaussian1D

# Local
from . import config

def get_version():
    try:
        return version("luxetc")
    except PackageNotFoundError:
        return "unknown"


__version__ = get_version()

from .hardware import Hardware  # noqa: E402, F401
from .uvchannel import UVChannel	# noqa: E402, F401
from .optchannel import OptChannel 	# noqa: E402, F401
from .nirchannel import NIRChannel	# noqa: E402, F401
from .orbit import Orbit  # noqa: E402, F401
from . import spec_etc_utils
#from .phoenix import SED  # noqa: E402, F401
#from .utils import *  # noqa: E402, F401, F403


class LuxETC(object):
    """Holds information and methods for the full Lux system."""

    def __init__(self, conf="REQ"):
    	
    	# Load all subclasses
        self.Orbit = Orbit()
        self.Hardware = Hardware()
        self.UVChannel = UVChannel(conf=conf)
        self.OptChannel = OptChannel(conf=conf)
        self.NIRChannel = NIRChannel(conf=conf)
        
        # Setup configuration
        if conf=="CBE":
            print("You are using Current Best Estimates (CBE) of system performance. Please make sure this is intended (Lux must achieve science objects at the requirements level).")
        elif conf=="REQ":
            print("You are using Requirements-level performace (REQ). This is default behavior.")
        elif conf=="REQ-FAVG":
            print("You are using Requirements-level performace (REQ) with FOV-averaged aperture fractions. This is not recommended, but is available for testing purposes.")
        else:
            raise ValueError("No such configuration setting: %s" % conf)
        self.conf = conf
            
    def __repr__(self):
        return "Lux Satellite"

    def _repr_html_(self):
        return "Lux Satellite"
        
    def zodi_counts(self, filt, zodi_level="avg"):
        """Calculates zodiacal light rate (per pixel) for a given filter""" 
        
        f = fits.open(config.DATADIR + "zodi_avg.fits")
        z = f[1].data["Photon Flux"] / u.s / u.cm / u.cm / u.AA / u.arcsec / u.arcsec
        if (zodi_level=="high"):
            z *= 1.73
        elif (zodi_level=="low"):
            z /= 1.73
        	
        if filt in config.LUXFILTS["UVCHAN"]:
            zfl = z * self.UVChannel.Aeff[filt] * self.UVChannel.pixel_scale**2
        elif filt in config.LUXFILTS["OPTCHAN"]:
            zfl = z * self.OptChannel.Aeff[filt] * self.OptChannel.pixel_scale**2
        elif filt in config.LUXFILTS["NIRCHAN"]:
            zfl = z * self.NIRChannel.Aeff[filt] * self.NIRChannel.pixel_scale**2
        else:
        	raise ValueError("No such filter in set: %s" % filt)
            
        return np.trapezoid(zfl, x=config.WAV.to('AA')) * u.electron * u.pixel**2
        
    def airglow_counts(self, filt, airglow_level="avg"):
        """Calculates airglow rate (per pixel) for a given filter"""
        
        ag = np.zeros(len(config.WAV)) * u.erg / u.s / u.cm / u.cm / u.AA / u.arcsec / u.arcsec
        if airglow_level=="low":
            ag[147] = 1.5e-18 * u.erg / u.s / u.cm / u.cm / u.AA / u.arcsec / u.arcsec
        elif airglow_level=="avg":
            ag[147] = 1.5e-16 * u.erg / u.s / u.cm / u.cm / u.AA / u.arcsec / u.arcsec
        elif airglow_level=="high":
            ag[147] = 3.0e-16 * u.erg / u.s / u.cm / u.cm / u.AA / u.arcsec / u.arcsec
           
        E = const.h * const.c / config.WAV.to('AA')
        
        if filt in config.LUXFILTS["UVCHAN"]:
            photon_spectrum = ag * self.UVChannel.Aeff[filt] * self.UVChannel.pixel_scale**2 / E.to('erg')
            return np.trapezoid(photon_spectrum, x=config.WAV.to('AA')) * u.pixel * u.pixel * u.electron
        elif filt in config.LUXFILTS["OPTCHAN"]:
            photon_spectrum = ag * self.OptChannel.Aeff[filt] * self.OptChannel.pixel_scale**2 / E.to('erg')
            return np.trapezoid(photon_spectrum, x=config.WAV.to('AA')) * u.pixel * u.pixel * u.electron
        elif filt in config.LUXFILTS["NIRCHAN"]:
            photon_spectrum = ag * self.NIRChannel.Aeff[filt] * self.NIRChannel.pixel_scale**2 / E.to('erg')
            return np.trapezoid(photon_spectrum, x=config.WAV.to('AA')) * u.pixel * u.pixel * u.electron
        else:
        	raise ValueError("No such filter in set: %s" % filt)
        
    def source_counts(self, source, filt):
        """Calculates source count rate (per pixel) for a given filter"""
        
        E = const.h * const.c / config.WAV.to('AA')
        
        if filt in config.LUXFILTS["UVCHAN"]:
            photon_spectrum = source * self.UVChannel.Aeff[filt] / E.to('erg')
            return self.UVChannel.aperture_fraction[filt] * \
                    np.trapezoid(photon_spectrum, x=config.WAV.to('AA')) * u.electron
        elif filt in config.LUXFILTS["OPTCHAN"]:
            photon_spectrum = source * self.OptChannel.Aeff[filt] / E.to('erg')
            return self.OptChannel.aperture_fraction[filt] * \
                    np.trapezoid(photon_spectrum, x=config.WAV.to('AA')) * u.electron  
        elif filt in config.LUXFILTS["NIRCHAN"]:
            photon_spectrum = source * self.NIRChannel.Aeff[filt] / E.to('erg')
            return self.NIRChannel.aperture_fraction[filt] * \
                    np.trapezoid(photon_spectrum, x=config.WAV.to('AA')) * u.electron        
        else:
        	raise ValueError("No such filter in set: %s" % filt)

    def get_snr(self, source, texps={}, zodi_level="avg", airglow_level="avg"):
        """Calculate SNR given source, filter, and texp"""
        
        SNR = {}
        
        # Loop through filters
        for filt, texp in texps.items():
        
            # Source counts
            src_cts = self.source_counts(source, filt) * texp * u.s
            
            # Zodi noise
            zodi_noise = self.zodi_counts(filt, zodi_level) * texp * u.s
            
            # Airglow noise
            aglow_noise = self.airglow_counts(filt, airglow_level) * texp * u.s
            
            if filt in config.LUXFILTS["UVCHAN"]:
                # Dark noise
                dark_noise = self.UVChannel.background_dark_rate * texp * u.s
            
                # Number of frames
                Nframes = texp * u.s / self.UVChannel.integration_time
            
                temp = src_cts / np.sqrt(src_cts + self.UVChannel.aperture_area[filt] * \
                             (zodi_noise + dark_noise + aglow_noise + 
                             Nframes * self.UVChannel.background_read_noise**2 / u.electron))
                SNR[filt] = temp.value
                
            elif filt in config.LUXFILTS["OPTCHAN"]:
                # Dark noise
                dark_noise = self.OptChannel.background_dark_rate * texp * u.s
            
                # Number of frames
                Nframes = texp * u.s / self.OptChannel.integration_time
            
                temp = src_cts / np.sqrt(src_cts + self.OptChannel.aperture_area[filt] * \
                             (zodi_noise + dark_noise + aglow_noise +
                             Nframes * self.OptChannel.background_read_noise**2 / u.electron))
                SNR[filt] = temp.value
                
            elif filt in config.LUXFILTS["NIRCHAN"]:
                # Dark noise
                dark_noise = self.NIRChannel.background_dark_rate * texp * u.s
            
                # Number of frames
                Nframes = texp * u.s / self.NIRChannel.integration_time
            
                temp = src_cts / np.sqrt(src_cts + self.NIRChannel.aperture_area[filt] * \
                             (zodi_noise + dark_noise + aglow_noise +
                             Nframes * self.NIRChannel.background_read_noise**2 / u.electron))
                SNR[filt] = temp.value
                
            else:
                raise ValueError("No such filter in set: %s" % filt)

        return SNR
        
    def get_texp(self, source, snrs={}, zodi_level="avg", airglow_level="avg"):
        """Calculate required exposure times given source, filter, and SNR"""
        
        texps = {}
        
        # Loop through filters
        for filt, SNR in snrs.items():
        
            # Source rate
            Rsrc = self.source_counts(source, filt)
            
            # Zodi rate
            Rzodi = self.zodi_counts(filt, zodi_level)
            
            # Airglow rate
            Raglow = self.airglow_counts(filt, airglow_level)
            
            if filt in config.LUXFILTS["UVCHAN"]:
                Npix = self.UVChannel.aperture_area[filt]
                Rdark = self.UVChannel.background_dark_rate
                RN = self.UVChannel.background_read_noise
                Tframe = self.UVChannel.integration_time
            elif filt in config.LUXFILTS["OPTCHAN"]:
                Npix = self.OptChannel.aperture_area[filt]
                Rdark = self.OptChannel.background_dark_rate
                RN = self.OptChannel.background_read_noise
                Tframe = self.OptChannel.integration_time
            elif filt in config.LUXFILTS["NIRCHAN"]:
                Npix = self.NIRChannel.aperture_area[filt]
                Rdark = self.NIRChannel.background_dark_rate
                RN = self.NIRChannel.background_read_noise
                Tframe = self.NIRChannel.integration_time
            else:
                raise ValueError("No such filter in set: %s" % filt)

            temp1 = SNR**2 * (Rsrc + Npix * Rzodi + Npix * Raglow + Npix * Rdark + Npix * RN**2 / Tframe / u.electron) / Rsrc**2    
            texps[filt] = temp1.value
            
        return texps         
        
    def get_limmag(self, filts={}, zodi_level="avg", airglow_level="avg"):
        """Calculate limiting magnitude for given filter, texp, and SNR"""
        
        limmags = {}
        
        # Loop through filters
        for filt, (Texp, SNR) in filts.items():
        
            # Zodi rate
            Rzodi = self.zodi_counts(filt, zodi_level)
            
            # Airglow rate
            Raglow = self.airglow_counts(filt, airglow_level)
            
            if filt in config.LUXFILTS["UVCHAN"]:
                Npix = self.UVChannel.aperture_area[filt]
                Rdark = self.UVChannel.background_dark_rate
                RN = self.UVChannel.background_read_noise
                Tframe = self.UVChannel.integration_time
                ZP = self.UVChannel.zero_point[filt]
            elif filt in config.LUXFILTS["OPTCHAN"]:
                Npix = self.OptChannel.aperture_area[filt]
                Rdark = self.OptChannel.background_dark_rate
                RN = self.OptChannel.background_read_noise
                Tframe = self.OptChannel.integration_time
                ZP = self.OptChannel.zero_point[filt]
            elif filt in config.LUXFILTS["NIRCHAN"]:
                Npix = self.NIRChannel.aperture_area[filt]
                Rdark = self.NIRChannel.background_dark_rate
                RN = self.NIRChannel.background_read_noise
                Tframe = self.NIRChannel.integration_time
                ZP = self.NIRChannel.zero_point[filt]
            else:
                raise ValueError("No such filter in set: %s" % filt)
               
            a = Texp / (SNR**2)
            b = -1
            ct = -Npix * (Rzodi + Raglow + Rdark + RN**2 / Tframe / u.electron)
            c = ct.value    
            
            Rsrc = (-b + np.sqrt(b**2 - 4. * a * c)) / 2. / a
            limmags[filt] = -2.5 * np.log10(Rsrc) + ZP   
               
        return limmags   
  
        
        



"""Functions for the spectroscopic mode of the Lux ETC"""

import numpy as np
from astropy import units as u
from specutils import Spectrum1D
from specutils.manipulation import FluxConservingResampler
from astropy.io import ascii
from astropy.nddata import StdDevUncertainty
from specutils import Spectrum
from specutils.manipulation import gaussian_smooth
from specutils import SpectralRegion
from specutils.manipulation import extract_region
# Standard library
from dataclasses import dataclass

# Local
from . import config
from . import hardware
from . import nirchannel
from . import uvchannel
from . import optchannel

from .optchannel import OptChannel
from .uvchannel import UVChannel
from .nirchannel import NIRChannel

optchannel = OptChannel()
uvchannel = UVChannel()
nirchannel = NIRChannel()

def luxreturnwave():
    return config.WAV

def luxreturnaeffoptspec():
    print('hello')
    yo = optchannel.Aeff_optspec
    return yo
    #return optchannel.OptChannel.Aeff_optspec()



def luxspecformat(inspec_path,mode="opt"):
    """for an input spectrum, will spit out a spectrum rebinned and 
    smoothed to Lux dispersion and resolution """

    #input spectrum should be an ascii file where first two columns are ang, flux (erg/cm**2/A)
    
    data = ascii.read(inspec_path)

    # Extract columns and assign units
    wavelength = data['col1'] * u.AA
    flux = data['col2'] * u.erg / (u.cm**2 * u.s * u.AA)

    #create spectrum object, find wmin and wmax
    spec = Spectrum(spectral_axis=wavelength, flux=flux)
    w_min = spec.spectral_axis.min()
    w_max = spec.spectral_axis.max()

    #initialize resampler
    resampler = FluxConservingResampler()
    
    #which mode are we in, assign wavelength range
    if mode=="opt":
        lux_wmin = 4000. * u.AA
        lux_wmax = 8000. * u.AA
        #Create new grid from w_min to w_max with 10A steps
        #Assuming R~100 at 5000A.  
        #10A/pix gives us 5 pixel per resolution element
        pixscale = 10. #10A per pixel
        sampling_factor = 5.
        new_grid = np.arange(w_min.value, w_max.value, pixscale) * u.AA

        #this returns a new spectrum object on the 10A (pixscale) grid
        spec_rebinned = resampler(spec, new_grid)
        
        # Check the results
        #print(f"Original bins: {len(spec.spectral_axis)}")
        #print(f"New bins: {len(spec_rebinned.spectral_axis)}")

        dw = np.diff(spec_rebinned.spectral_axis).mean()

        #find sigma and smooth spectrum
        sigma_angstrom = pixscale * sampling_factor * u.AA
        sigma_pixels = (sigma_angstrom / dw).decompose().value
        smoothed_spec = gaussian_smooth(spec_rebinned, stddev=sigma_pixels)

        #cut out relevant wavelength range
        region = SpectralRegion(lux_wmin, lux_wmax)

        # Extract the sub-spectrum from your original object
        sub_smoothedspectrum = extract_region(smoothed_spec, region)

    elif mode=="uv":
        lux_wmin = 2000. * u.AA
        lux_wmax = 4000. * u.AA
        #Create new grid from w_min to w_max with 10A steps
        #Assuming R~100 at 3000A.  
        #6A/pix gives us 5 pixel per resolution element
        pixscale = 6. #6A per pixel
        sampling_factor = 5.
        
        new_grid = np.arange(w_min.value, w_max.value, pixscale) * u.AA

        #this returns a new spectrum object on the 6A (pixscale) grid
        spec_rebinned = resampler(spec, new_grid)
        dw = np.diff(spec_rebinned.spectral_axis).mean()

         #find sigma and smooth spectrum
        sigma_angstrom = pixscale * sampling_factor * u.AA
        sigma_pixels = (sigma_angstrom / dw).decompose().value
        smoothed_spec = gaussian_smooth(spec_rebinned, stddev=sigma_pixels)

        #cut out relevant wavelength range
        region = SpectralRegion(lux_wmin, lux_wmax)

        # Extract the sub-spectrum from your original object
        sub_smoothedspectrum = extract_region(smoothed_spec, region)
       

    
    elif mode=="nir":
        raise ValueError("NIR spectroscopic mode is not implemented yet")

    else:
        raise ValueError("No such mode in set: %s" % mode)

    return sub_smoothedspectrum


def luxspec_cnts(luxspec_in, mode='opt'):

    #this function will take an input spectrum in lux resolution 
    #and pixel scale (from luxspecformat) and multiply it by 
    #the effective area.
    
    #input spectroscopic units should be u.erg / (u.cm**2 * u.s * u.AA)
    #this is not being checked right now

    #input should be the rebinned, smoothed spectrum made by luxspecformat

    
    # Define the target unit
    target_unit = u.photon / (u.cm**2 * u.s * u.AA)

    # We use the spectral_density equivalency, passing the spectral axis 
    # so the code knows how to calculate energy-per-photon at each point.
    spec_photons = luxspec_in.with_flux_unit(
        target_unit, 
        equivalencies=u.spectral_density(luxspec_in.spectral_axis)
    )

    resampler = FluxConservingResampler()
    
    #which mode are we in
    if mode=="opt":
        #get the optical effective area into specutils format
        Aeff_optspec = Spectrum(flux=optchannel.Aeff_optspec,
                     spectral_axis=config.WAV)

        # Match the units of the spectral axis before resampling
        Aeff_optspec_converted = \
            Aeff_optspec.with_spectral_axis_unit(spec_photons.spectral_axis.unit)

        
        # Resample Aeff_optspec to match the grid of luxspec_in
        Aeff_optspec_matched = \
            resampler(Aeff_optspec_converted,spec_photons.spectral_axis)

        src_cts = spec_photons * Aeff_optspec_matched
    
    elif mode=="uv":

        #get the optical effective area into specutils format
        Aeff_uvspec = Spectrum(flux=uvchannel.Aeff_uvspec,
                     spectral_axis=config.WAV)

        # Match the units of the spectral axis before resampling
        Aeff_uvspec_converted = \
            Aeff_uvspec.with_spectral_axis_unit(spec_photons.spectral_axis.unit)

        # Resample Aeff_uvspec to match the grid of luxspec_in
        Aeff_uvspec_matched = \
            resampler(Aeff_uvspec_converted,spec_photons.spectral_axis)

        src_cts = spec_photons * Aeff_uvspec_matched
   
    elif mode=="nir":
        raise ValueError("NIR spectroscopic mode is not implemented yet")

    else:
        raise ValueError("No such mode in set: %s" % mode)


    return src_cts

def rebin_SNR_spectrum(wavelengths, snr_per_pixel, factor=5):
    # Determine the number of new bins
    n_bins = len(wavelengths) // factor

    # Trim the arrays to be exactly divisible by the factor
    w_trimmed = wavelengths[:n_bins * factor]
    s_trimmed = snr_per_pixel[:n_bins * factor]

    # Average the wavelengths within each bin
    new_wavelength = w_trimmed.reshape(-1, factor).mean(axis=1)

    # Calculate the new SNR: Average SNR in the bin * sqrt(factor)
    new_snr = s_trimmed.reshape(-1, factor).mean(axis=1) * np.sqrt(factor)

    return new_wavelength, new_snr

def add_lux_noise(luxspec,snrperpix):

    #calculate per-pixel sigma
    #sigma(lambda) = flux(lambda) / SNR(lambda)
    flux_vals = luxspec.flux.value
    sigma_array = flux_vals / snrperpix.value

    # Generate noise scaled by the local sigma
    # We multiply a standard normal distribution by our sigma array
    noise = np.random.normal(0, 1, size=flux_vals.shape) * sigma_array

    # Create the new degraded spectrum
    degraded_flux = (flux_vals + noise) * luxspec.flux.unit
    uncertainty = StdDevUncertainty(sigma_array * luxspec.flux.unit)

    degraded_spectrum = Spectrum(
        spectral_axis=luxspec.spectral_axis, 
        flux=degraded_flux 
        #uncertainty=uncertainty
    )

    return degraded_spectrum

def luxspec_snr(inspec_path,texp,mode='opt'):

    exptime = texp * u.s

    Nframes = exptime / (config.TFRAME * u.s)

    #print(Nframes)
    #this is the number of pixels we are summing in spatial direction
    #think about implementing this in a smarter way later.
    Nspatial = 5. 

    #this is for converting from SNR/pixel to SNR/resolution element
    #needs to be implemented better, but is how we have set up luxspecformat
    Nspectral = 5. 

    gain = 1. * u.electron / u.photon

        
    #read in input spectrum and put into lux format
    luxspec = luxspecformat(inspec_path,mode=mode)

    #calculate spec counts per second and multiply by effective area.
    luxspec_counts = luxspec_cnts(luxspec, mode=mode)
    
    #which mode are we in
    if mode=="opt":
    #read in input spectrum and put into lux format
        #this really needs to be defined more universally, is in luxspecformat
        pixscale_opt = 10. * u.AA
        
        
        
        dark_noise = optchannel.background_dark_rate * texp * u.s
        dark_noise_withspatial = dark_noise * Nspatial 

        
        
        phot_elec = luxspec_counts * (exptime) * (pixscale_opt) * gain

        #calculate SNR per pixel
        SNRperpix = phot_elec.flux / \
            np.sqrt(phot_elec.flux + dark_noise_withspatial + Nframes * Nspatial * optchannel.background_read_noise**2 / u.electron ) 


        #calculate SNR per resolution element and get the wavelength array too
        SNRperres_wave, SNRperres = \
            rebin_SNR_spectrum(luxspec.spectral_axis, SNRperpix, factor=5)

        #calculate output spectrum with noise
        observed_spectrum = add_lux_noise(luxspec,SNRperpix)
    
    elif mode=="uv":

        #this really needs to be defined more universally, is in luxspecformat
        pixscale_uv = 6. * u.AA

        #Nframes = exptime / LETC.UVChannel.integration_time

        dark_noise = uvchannel.background_dark_rate * texp * u.s
        dark_noise_withspatial = dark_noise * Nspatial 

        phot_elec = luxspec_counts * (exptime) * (pixscale_uv) * gain

        #calculate SNR per pixel
        SNRperpix = phot_elec.flux / \
            np.sqrt(phot_elec.flux + dark_noise_withspatial + Nframes * Nspatial * uvchannel.background_read_noise**2 / u.electron ) 


        #calculate SNR per resolution element and get the wavelength array too
        SNRperres_wave, SNRperres = \
            rebin_SNR_spectrum(luxspec.spectral_axis, SNRperpix, factor=5)

        #calculate output spectrum with noise
        observed_spectrum = add_lux_noise(luxspec,SNRperpix)
    
    elif mode=="nir":
        raise ValueError("NIR spectroscopic mode is not implemented yet")

    else:
        raise ValueError("No such mode in set: %s" % mode)


    #1. return the spectrum rebinned and smoothed to Lux resolution.
    return luxspec, observed_spectrum, SNRperpix, SNRperres, SNRperres_wave



"""Holds metadata and methods on Lux Optical Channel"""

# Standard library
from dataclasses import dataclass

# Third-party
import astropy.units as u
import astropy.io.fits as fits
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

# Local
from . import config
from . import hardware
#from . import PANDORASTYLE

@dataclass
class OptChannel():
    """
    Holds information on the Lux Optical Channel
    """
    
    def __init__(self, conf="CBE"):
        
        # Set configuration
        self.conf = conf
        
        # Set common telescope properties
        self.hardware = hardware.Hardware()

        Aeff = self.optical_Aeff()
        self.Aeff = {"g": Aeff[0],
                     "r": Aeff[1],
                     "i": Aeff[2]}

        Aeff_optspec = self.optical_spec_Aeff()
        self.Aeff_optspec = Aeff_optspec
        
    def __repr__(self):
        return "OptChannel"
        
    @property
    def name(self):
        return "OPTCHAN"

    @property
    def shape(self):
        """Shape of the detector in pixels"""
        return (4096, 4096)
        ### Need to verify what portion of the detector is actually read out

    @property
    def pixel_scale(self):
        """Pixel scale of the detector"""
        return 1.08 * u.arcsec / u.pixel

    @property
    def pixel_size(self):
        """Size of a pixel"""
        return 9.0 * u.um / u.pixel

    @property
    def bits_per_pixel(self):
        """Number of bits per pixel"""
        return 16 * u.bit / u.pixel

    @property
    def naxis1(self):
        """WCS's are COLUMN major, so naxis1 is the number of columns"""
        return self.shape[1] * u.pixel

    @property
    def naxis2(self):
        """WCS's are COLUMN major, so naxis2 is the number of rows"""
        return self.shape[0] * u.pixel

    @property
    def background_dark_rate(self):
        """Detector background dark current rate"""
        if self.conf == "CBE":
            return 0.04 * u.electron / u.second
        elif self.conf == "REQ-FAVG":
            return 0.10 * u.electron / u.second
        elif self.conf == "REQ":
            return 0.10 * u.electron / u.second
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")
        
    @property
    def background_read_noise(self):
        """Detector background read noise"""
        if self.conf == "CBE":
            return 2.3 * u.electron
        elif self.conf == "REQ-FAVG":
            return 3.3 * u.electron
        elif self.conf == "REQ":
            return 3.3 * u.electron
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")
		
    @property
    def integration_time(self):
        """Integration time"""
        return 60.0 * u.second
        
    @property
    def aperture_area(self):
        """Area to sum for aperture photometry"""
        if self.conf == "CBE":
            return {"g": 22.2,
                    "r": 22.2,
                    "i": 22.2}
        elif self.conf == "REQ-FAVG":
            return {"g": 430.1,
                    "r": 430.1,
                    "i": 430.1}
        elif self.conf == "REQ":
            return {"g": 34.8,
                    "r": 34.8,
                    "i": 34.8}
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")
        
    @property
    def aperture_fraction(self):
        """Fraction of PSF included in aperture_area"""
        if self.conf == "CBE":
            return {"g": 0.714,
                    "r": 0.714,
                    "i": 0.714}
        elif self.conf == "REQ-FAVG":
            return {"g": 0.714,
                    "r": 0.714,
                    "i": 0.714}
        elif self.conf == "REQ":
            return {"g": 0.714,
                    "r": 0.714,
                    "i": 0.714}
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")

    @property
    def info(self):
        return pd.DataFrame(
            {
                "Detector Size": f"({self.naxis1.value.astype(int)}, {self.naxis2.value.astype(int)})",
                "Pixel Scale": f"{self.pixel_scale.value} {self.pixel_scale.unit.to_string('latex')}",
                "Pixel Size": f"{self.pixel_size.value} {self.pixel_size.unit.to_string('latex')}",
                "Read Noise": f"{self.background_read_noise.value} {self.background_read_noise.unit.to_string('latex')}",
                "Dark Noise": f"{self.background_dark_rate.value} {self.background_dark_rate.unit.to_string('latex')}",
                "Integration Time": f"{self.integration_time.value} {self.integration_time.unit.to_string('latex')}",
            },
            index=[0],
        ).T.rename({0: "OPTCHAN"}, axis="columns")
        
    @property
    def dichroic_throughput(self):
    	"""Dichroic Throughput"""
    	f = fits.open(config.DATADIR + "Optical_Dichroic.fits")
    	g = fits.open(config.DATADIR + "UV_Dichroic.fits")
    	return f[1].data["Reflectance"] * g[1].data["Transmission"]
    	
    @property
    def aft_optics_throughput(self):
    	"""Throughput of the 8 aft optics (AR coatings)"""
    	f = fits.open(config.DATADIR + "Aft_Optical_Throughput.fits")
    	return f[1].data["Throughput"]

    @property
    def optical_prisms_throughput(self):
        """Throughput of the prisms.  Only a guesstimate right now"""
        optprism_throughput = 0.5
        optical_prism_throughput = np.full(config.WAV.shape,optprism_throughput)
        return optical_prism_throughput
    
    @property
    def optical_detector_qe(self):
    	"""Quantum Efficiency of Optical Detector"""
    	f = fits.open(config.DATADIR + "Optical_Detector_QE.fits")
    	return f[1].data["QE"]


    def optical_spec_Aeff(self):
        """ Effective area of optical spec mode"""
        Aeff_optspec = 0.25 * np.pi * \
                       np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                       self.hardware.fore_throughput * \
                       self.dichroic_throughput * \
                       self.aft_optics_throughput * \
                       self.optical_prisms_throughput * \
                       self.optical_detector_qe

        if self.conf == "CBE":
            return Aeff_optspec
        elif self.conf == "REQ":
            return Aeff_optspec
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")

    
    def optical_Aeff(self):
        """Effective area for each filter"""
        
        # g-band
        f = fits.open(config.DATADIR + "LSST_g_filter.fits")
        filter_throughput = f[1].data["Throughput"]
        Aeff_g = 0.25 * np.pi * \
                  np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                  self.hardware.fore_throughput * \
                  self.dichroic_throughput * \
                  self.aft_optics_throughput * \
                  filter_throughput * \
                  self.optical_detector_qe
        
        #r-band
        f = fits.open(config.DATADIR + "LSST_r_filter.fits")
        filter_throughput = f[1].data["Throughput"]
        Aeff_r = 0.25 * np.pi * \
                  np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                  self.hardware.fore_throughput * \
                  self.dichroic_throughput * \
                  self.aft_optics_throughput * \
                  filter_throughput * \
                  self.optical_detector_qe
                  
        #i-band
        f = fits.open(config.DATADIR + "LSST_i_filter.fits")
        filter_throughput = f[1].data["Throughput"]
        Aeff_i = 0.25 * np.pi * \
                  np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                  self.hardware.fore_throughput * \
                  self.dichroic_throughput * \
                  self.aft_optics_throughput * \
                  filter_throughput * \
                  self.optical_detector_qe
                  
        if self.conf == "CBE":
            return (Aeff_g, Aeff_r, Aeff_i)
        elif self.conf == "REQ-FAVG":
            return (Aeff_g*0.85, Aeff_r*0.85, Aeff_i*0.85)
        elif self.conf == "REQ":
            return (Aeff_g*0.85, Aeff_r*0.85, Aeff_i*0.85)
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")
		    	
    @property
    def zero_point(self):
        """Photometric zeropoint"""
        if self.conf == "CBE":
            return {"g": 22.214,
                "r": 22.034,
                "i": 21.314}
        elif self.conf == "REQ-FAVG":
            return {"g": 22.038,
                "r": 21.858,
                "i": 21.138}
        elif self.conf == "REQ":
            return {"g": 22.038,
                "r": 21.858,
                "i": 21.138}
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")



"""Holds basic metadata on Lux orbit"""

# Standard library
from dataclasses import dataclass

# Third-party
import astropy.units as u


@dataclass
class Orbit:
    """Holds basic metadata on the orbit of Lux satellites"""

    period: float = 96.54 * u.minute  # circular orbit @ 600 km

    def __repr__(self):
        return "Lux Orbit"


import os

import numpy as np
import astropy.units as u

PACKAGEDIR = os.path.abspath(os.path.dirname(__file__))
DATADIR = "/".join(PACKAGEDIR.split("/")) + "/data/"

WAV = np.linspace(100., 1600., 1501) * u.nm

LUXFILTS = {"UVCHAN": ["uv2", "uv1", "u"],
            "OPTCHAN": ["g", "r", "i"],
            "NIRCHAN": ["z", "y", "j"]}
            
TFRAME = 60.


"""Holds metadata and methods on Lux NIR Channel"""

# Standard library
from dataclasses import dataclass

# Third-party
import astropy.units as u
import astropy.io.fits as fits
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

# Local
from . import config
from . import hardware
#from . import PANDORASTYLE

@dataclass
class NIRChannel():
    """
    Holds information on the Lux NIR Channel
    """
    
    def __init__(self, conf="CBE"):
        
        # Set configuration
        self.conf = conf
        
        # Set common telescope properties
        self.hardware = hardware.Hardware()

        Aeff = self.nir_Aeff()
        self.Aeff = {"z": Aeff[0],
                     "y": Aeff[1],
                     "j": Aeff[2]}

        Aeff_nirspec = self.nir_spec_Aeff()
        self.Aeff_nirspec = Aeff_nirspec

        
    def __repr__(self):
        return "NIRChannel"
        
    @property
    def name(self):
        return "NIRCHAN"

    @property
    def shape(self):
        """Shape of the detector in pixels"""
        return (4096, 4096)
        ### Need to verify what portion of the detector is actually read out

    @property
    def pixel_scale(self):
        """Pixel scale of the detector"""
        return 1.20 * u.arcsec / u.pixel

    @property
    def pixel_size(self):
        """Size of a pixel"""
        return 10.0 * u.um / u.pixel

    @property
    def bits_per_pixel(self):
        """Number of bits per pixel"""
        return 16 * u.bit / u.pixel

    @property
    def naxis1(self):
        """WCS's are COLUMN major, so naxis1 is the number of columns"""
        return self.shape[1] * u.pixel

    @property
    def naxis2(self):
        """WCS's are COLUMN major, so naxis2 is the number of rows"""
        return self.shape[0] * u.pixel

    @property
    def background_dark_rate(self):
        """Detector background dark current rate"""
        if self.conf == "CBE":
            return 0.016 * u.electron / u.second
        elif self.conf == "REQ-FAVG":
            return 0.05 * u.electron / u.second
        elif self.conf == "REQ":
            return 0.05 * u.electron / u.second
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")
        
    @property
    def background_read_noise(self):
        """Detector background read noise"""
        if self.conf == "CBE":
            return 5.5 * u.electron
        elif self.conf == "REQ-FAVG":
            return 7.5 * u.electron
        elif self.conf == "REQ":
            return 7.5 * u.electron
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")
		
    @property
    def integration_time(self):
        """Integration time"""
        return 60.0 * u.second
        
    @property
    def aperture_area(self):
        """Area to sum for aperture photometry"""
        if self.conf == "CBE":
            return {"z": 17.9,
                    "y": 17.9,
                    "j": 17.9}
        elif self.conf == "REQ-FAVG":
            return {"z": 353.0,
                    "y": 353.0,
                    "j": 353.0}
        elif self.conf == "REQ":
            return {"z": 28.2,
                    "y": 28.2,
                    "j": 28.2}
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")
        
    @property
    def aperture_fraction(self):
        """Fraction of PSF included in aperture_area"""
        if self.conf == "CBE":
            return {"z": 0.714,
                    "y": 0.714,
                    "j": 0.714}
        elif self.conf == "REQ-FAVG":
            return {"z": 0.714,
                    "y": 0.714,
                    "j": 0.714}
        elif self.conf == "REQ":
            return {"z": 0.714,
                    "y": 0.714,
                    "j": 0.714}
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")

    @property
    def info(self):
        return pd.DataFrame(
            {
                "Detector Size": f"({self.naxis1.value.astype(int)}, {self.naxis2.value.astype(int)})",
                "Pixel Scale": f"{self.pixel_scale.value} {self.pixel_scale.unit.to_string('latex')}",
                "Pixel Size": f"{self.pixel_size.value} {self.pixel_size.unit.to_string('latex')}",
                "Read Noise": f"{self.background_read_noise.value} {self.background_read_noise.unit.to_string('latex')}",
                "Dark Noise": f"{self.background_dark_rate.value} {self.background_dark_rate.unit.to_string('latex')}",
                "Integration Time": f"{self.integration_time.value} {self.integration_time.unit.to_string('latex')}",
            },
            index=[0],
        ).T.rename({0: "NIRCHAN"}, axis="columns")
        
    @property
    def dichroic_throughput(self):
        """Dichroic Throughput"""
        f = fits.open(config.DATADIR + "UV_Dichroic.fits")
        g = fits.open(config.DATADIR + "Optical_Dichroic.fits")
        return f[1].data["Transmission"] * g[1].data["Transmission"]

    @property
    def aft_optics_throughput(self):
        """Throughput of the 8 aft optics (AR coatings)"""
        f = fits.open(config.DATADIR + "Aft_NIR_Throughput.fits")
        return f[1].data["Throughput"]

    @property
    def nir_prisms_throughput(self):
        """Throughput of the prisms.  Only a guesstimate right now"""
        nirprism_throughput = 0.5
        nir_prism_throughput = np.full(config.WAV.shape,nirprism_throughput)
        return nir_prism_throughput
    
    @property
    def nir_detector_qe(self):
        """Quantum Efficiency of Optical Detector"""
        f = fits.open(config.DATADIR + "NIR_Detector_QE.fits")
        return f[1].data["QE"]
    	
    def nir_spec_Aeff(self):
        """ Effective area of optical spec mode"""
        Aeff_nirspec = 0.25 * np.pi * \
                       np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                       self.hardware.fore_throughput * \
                       self.dichroic_throughput * \
                       self.aft_optics_throughput * \
                       self.nir_prisms_throughput * \
                       self.nir_detector_qe

        if self.conf == "CBE":
            return Aeff_nirspec
        elif self.conf == "REQ":
            return Aeff_nirspec
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")

    def nir_Aeff(self):
        """Effective area for each filter"""
        
        # z-band
        f = fits.open(config.DATADIR + "LSST_z_filter.fits")
        filter_throughput = f[1].data["Throughput"]
        Aeff_z = 0.25 * np.pi * \
                  np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                  self.hardware.fore_throughput * \
                  self.dichroic_throughput * \
                  self.aft_optics_throughput * \
                  filter_throughput * \
                  self.nir_detector_qe
        
        # y-band
        f = fits.open(config.DATADIR + "Roman_y_filter.fits")
        filter_throughput = f[1].data["Throughput"]
        Aeff_y = 0.25 * np.pi * \
                  np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                  self.hardware.fore_throughput * \
                  self.dichroic_throughput * \
                  self.aft_optics_throughput * \
                  filter_throughput * \
                  self.nir_detector_qe
                  
        # j-band
        f = fits.open(config.DATADIR + "Roman_j_filter.fits")
        filter_throughput = f[1].data["Throughput"]
        Aeff_j = 0.25 * np.pi * \
                  np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                  self.hardware.fore_throughput * \
                  self.dichroic_throughput * \
                  self.aft_optics_throughput * \
                  filter_throughput * \
                  self.nir_detector_qe
                  
        if self.conf == "CBE":
            return (Aeff_z, Aeff_y, Aeff_j)
        elif self.conf == "REQ-FAVG":
            return (Aeff_z*0.85, Aeff_y*0.85, Aeff_j*0.85)
        elif self.conf == "REQ":
            return (Aeff_z*0.85, Aeff_y*0.85, Aeff_j*0.85)
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")

    @property
    def zero_point(self):
        """Photometric zeropoint"""
        if self.conf == "CBE":
            return {"z": 21.193,
                "y": 22.245,
                "j": 22.140}
        elif self.conf == "REQ-FAVG":
            return {"z": 21.017,
                "y": 22.069,
                "j": 21.964}
        elif self.conf == "REQ":
            return {"z": 21.017,
                "y": 22.069,
                "j": 21.964}
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")

        



"""Holds metadata and methods on Lux UV Channel"""

# Standard library
from dataclasses import dataclass

# Third-party
import astropy.units as u
import astropy.io.fits as fits
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

# Local
from . import config
from . import hardware
#from . import PANDORASTYLE

@dataclass
class UVChannel():
    """
    Holds information on the Lux UV Channel
    """
    
    def __init__(self, conf="REQ"):

        # Set configuration
        self.conf = conf
        
        # Set common telescope properties
        self.hardware = hardware.Hardware()

        # Set effective area
        Aeff = self.uv_Aeff()
        self.Aeff = {"uv2": Aeff[0],
                     "uv1": Aeff[1],
                     "u": Aeff[2]}

        Aeff_uvspec = self.uv_spec_Aeff()
        self.Aeff_uvspec = Aeff_uvspec

        
    def __repr__(self):
        return "UVChannel"
        
    @property
    def name(self):
        return "UVCHAN"

    @property
    def shape(self):
        """Shape of the detector in pixels"""
        return (4096, 4096)
        ### Need to verify what portion of the detector is actually read out

    @property
    def pixel_scale(self):
        """Pixel scale of the detector"""
        return 1.08 * u.arcsec / u.pixel

    @property
    def pixel_size(self):
        """Size of a pixel"""
        return 9.0 * u.um / u.pixel

    @property
    def bits_per_pixel(self):
        """Number of bits per pixel"""
        return 16 * u.bit / u.pixel

    @property
    def naxis1(self):
        """WCS's are COLUMN major, so naxis1 is the number of columns"""
        return self.shape[1] * u.pixel

    @property
    def naxis2(self):
        """WCS's are COLUMN major, so naxis2 is the number of rows"""
        return self.shape[0] * u.pixel

    @property
    def background_dark_rate(self):
        """Detector background dark current rate"""
        if self.conf == "CBE":
            return 0.04 * u.electron / u.second
        elif self.conf == "REQ-FAVG":
            return 0.10 * u.electron / u.second
        elif self.conf == "REQ":
            return 0.10 * u.electron / u.second
        else:
            raise ValueError("No such configuration setting: %s" % self.conf)

    @property
    def background_read_noise(self):
        """Detector background read noise"""
        if self.conf == "CBE":
            return 2.3 * u.electron
        elif self.conf == "REQ-FAVG":
            return 3.3 * u.electron
        elif self.conf == "REQ":
            return 3.3 * u.electron
        else:
            raise ValueError("No such configuration setting: %s" % self.conf)
		
    @property
    def integration_time(self):
        """Integration time"""
        return 60.0 * u.second
        
    @property
    def aperture_area(self):
        """Area to sum for aperture photometry"""
        if self.conf == "CBE":
            return {"uv2": 34.2,
                    "uv1": 34.2,
                    "u": 34.2}
        elif self.conf == "REQ-FAVG":
            return {"uv2": 430.1,
                    "uv1": 430.1,
                    "u": 430.1}
        elif self.conf == "REQ":
            return {"uv2": 51.2,
                    "uv1": 51.2,
                    "u": 51.2}
        else:
            raise ValueError("No such configuration setting: %s" % self.conf)
        
    @property
    def aperture_fraction(self):
        """Fraction of PSF included in aperture_area"""
        if self.conf == "CBE":
            return {"uv2": 0.709,
                    "uv1": 0.709,
                    "u": 0.709}
        elif self.conf == "REQ-FAVG":
            return {"uv2": 0.709,
                    "uv1": 0.709,
                    "u": 0.709}
        elif self.conf == "REQ":
            return {"uv2": 0.709,
                    "uv1": 0.709,
                    "u": 0.709}
        else:
            raise ValueError("No such configuration setting: %s" % self.conf)

    @property
    def info(self):
        return pd.DataFrame(
            {
                "Detector Size": f"({self.naxis1.value.astype(int)}, {self.naxis2.value.astype(int)})",
                "Pixel Scale": f"{self.pixel_scale.value} {self.pixel_scale.unit.to_string('latex')}",
                "Pixel Size": f"{self.pixel_size.value} {self.pixel_size.unit.to_string('latex')}",
                "Read Noise": f"{self.background_read_noise.value} {self.background_read_noise.unit.to_string('latex')}",
                "Dark Noise": f"{self.background_dark_rate.value} {self.background_dark_rate.unit.to_string('latex')}",
                "Integration Time": f"{self.integration_time.value} {self.integration_time.unit.to_string('latex')}",
            },
            index=[0],
        ).T.rename({0: "UVCHAN"}, axis="columns")
        
    @property
    def dichroic_throughput(self):
    	"""Dichroic Throughput"""
    	f = fits.open(config.DATADIR + "UV_Dichroic.fits")
    	return f[1].data["Reflectance"]
    	
    @property
    def aft_optics_throughput(self):
    	"""Throughput of the 6 aft optics (AR coatings)"""
    	f = fits.open(config.DATADIR + "Aft_UV_Throughput.fits")
    	return f[1].data["Throughput"]

    @property
    def uv_prisms_throughput(self):
        """Throughput of the prisms.  Only a guesstimate right now"""
        uvprism_throughput = 0.5
        uv_prism_throughput = np.full(config.WAV.shape,uvprism_throughput)
        return uv_prism_throughput
    
    @property
    def optical_detector_qe(self):
    	"""Quantum Efficiency of Optical Detector"""
    	f = fits.open(config.DATADIR + "Optical_Detector_QE.fits")
    	return f[1].data["QE"]
    	
    def uv_spec_Aeff(self):
        """ Effective area of optical spec mode"""
        Aeff_uvspec = 0.25 * np.pi * \
                       np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                       self.hardware.fore_throughput * \
                       self.dichroic_throughput * \
                       self.aft_optics_throughput * \
                       self.uv_prisms_throughput * \
                       self.optical_detector_qe

        if self.conf == "CBE":
            return Aeff_uvspec
        elif self.conf == "REQ":
            return Aeff_uvspec
        else:
            raise ValueError("Invalid configuration. Must be 'CBE' or 'REQ'.")

    def uv_Aeff(self):
        """Effective area for each filter"""
        
        # u-band
        f = fits.open(config.DATADIR + "LSST_u_filter.fits")
        filter_throughput = f[1].data["Throughput"]
        Aeff_u = 0.25 * np.pi * \
                  np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                  self.hardware.fore_throughput * \
                  self.dichroic_throughput * \
                  self.aft_optics_throughput * \
                  filter_throughput * \
                  self.optical_detector_qe
        
        #uv1-band
        f = fits.open(config.DATADIR + "UV1_filter.fits")
        filter_throughput = f[1].data["Throughput"]
        Aeff_uv1 = 0.25 * np.pi * \
                  np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                  self.hardware.fore_throughput * \
                  self.dichroic_throughput * \
                  self.aft_optics_throughput * \
                  filter_throughput * \
                  self.optical_detector_qe
                  
        #uv2-band (UVEX)
        f = fits.open(config.DATADIR + "UVEX_NUV_filter.fits")
        filter_throughput = f[1].data["Throughput"]
        Aeff_uv2 = 0.25 * np.pi * \
                  np.power(self.hardware.primary_mirror_effective_diameter,2) * \
                  self.hardware.fore_throughput * \
                  self.dichroic_throughput * \
                  self.aft_optics_throughput * \
                  filter_throughput * \
                  self.optical_detector_qe
                  
        if self.conf == "CBE":
            return (Aeff_uv2, Aeff_uv1, Aeff_u)
        elif self.conf == "REQ":
            return (Aeff_uv2*0.75, Aeff_uv1*0.75, Aeff_u*0.75)
        elif self.conf == "REQ-FAVG":
            return (Aeff_uv2*0.75, Aeff_uv1*0.75, Aeff_u*0.75)
        else:
            raise ValueError("No such configuration setting: %s" % self.conf)
		    	
    @property
    def zero_point(self):
        """Photometric zeropoint"""
        if self.conf == "CBE":
            return {"u": 21.193,
                    "uv1": 20.312,
                    "uv2": 20.296}
        elif self.conf == "REQ-FAVG":
            return {"u": 20.881,
                    "uv1": 20.000,
                    "uv2": 19.984}
        elif self.conf == "REQ":
            return {"u": 20.881,
                    "uv1": 20.000,
                    "uv2": 19.984}
        else:
            raise ValueError("No such configuration setting: %s" % self.conf)
